/*///////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	conf.c
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:
History:
	931120	extracted from onewlib.c
///////////////////////////////////////////////////////////////////////*/
#include "onew.h"

typedef struct {
	char		*name;
	char		*kname;
	RkEngine	*rk;
	KkEngine	*kk;
} Engine;

/*
 *	known engines
 */
RkEngine ONEW_WnnRomkan;
KkEngine ONEW_WnnKanakan;
RkEngine ONEW_CannaRk;
KkEngine ONEW_CannaKanakan;

Engine Engines[] = {
	{"none", "?"},
	{"wnn",  ONEW_KK_WNN,   &ONEW_WnnRomkan, &ONEW_WnnKanakan  },
	{"canna",ONEW_KK_CANNA, &ONEW_CannaRk,   &ONEW_CannaKanakan},
	0
};

/*
 *	current engines
 */
static int RomkanX;
static int RomkanN;
RkEngine *RomkanP(){ return Engines[RomkanX].rk; }
static int KanakanX;
static int KanakanN;
KkEngine *KanakanP(){return Engines[KanakanX].kk;}

#define RK_EXIST(rk)	((rk) && (rk->rk_ready))
#define KK_EXIST(kk)	((kk) && (kk->kk_ready))

static select_engine(){
	char *engine,*name,rkname[128],kkname[128];
	int ei;

	if( (engine = getenv("ONEW_ENGINE")) == 0 )
		engine = ONEW_ENGINE;
	switch( sscanf(engine,"%[^/]/%s",rkname,kkname) ){
		case 0: strcpy(rkname,"wnn");
		case 1: strcpy(kkname,rkname);
	}
	for( ei = 1; name = Engines[ei].name; ei++ ){
		if( strcmp(rkname,name) == 0 )
			if( RK_EXIST(Engines[ei].rk) ) RomkanX  = ei;
		if( strcmp(kkname,name) == 0 )
			if( KK_EXIST(Engines[ei].kk) ) KanakanX = ei;
	}

	for( ei = 1; name = Engines[ei].name; ei++ ){
		if( RK_EXIST(Engines[ei].rk) ){
			if( RomkanP() == 0 ) RomkanX  = ei;
			RomkanN++;
		}
		if( KK_EXIST(Engines[ei].kk) ){
			if( KanakanP()== 0 ) KanakanX = ei;
			KanakanN++;
		}
	}
}

romkan_select(){  if( RomkanX == 0 ) select_engine(); }
kanakan_select(){ if( KanakanX == 0 ) select_engine(); }

static	char *kkEngineID(){ return Engines[KanakanX].kname; }
	char *rkEngineID(){ return Engines[RomkanX].kname; }

static char *next_kanakan(){
	KkEngine *kk;
	char *name;
	int ei;

	ei = KanakanX + 1;
	if( (name=Engines[ei].name) && (kk=Engines[ei].kk) && KK_EXIST(kk) )
		KanakanX = ei;
	else	for( ei = 1; name = Engines[ei].name; ei++ )
			if( (kk=Engines[ei].kk) && KK_EXIST(kk) ){
				KanakanX = ei;
				break;
			}
	return name;
}
static char *next_romkan(){
	RkEngine *rk;
	char *name;
	int ei;

	ei = RomkanX + 1;
	if( (name=Engines[ei].name) && (rk=Engines[ei].rk) && RK_EXIST(rk) )
		RomkanX = ei;
	else	for( ei = 1; name = Engines[ei].name; ei++ )
			if( (rk=Engines[ei].rk) && RK_EXIST(rk) ){
				RomkanX = ei;
				break;
			}
	return name;
}
switch_rkserv(){
	char help[128];
	char *romkan;
	int kmode;

	if( RomkanP() == 0 )
		romkan_select();
	if( RomkanN <= 1)
		return;

	kmode = ONEW_kanamode;
	if( romkan = next_romkan() ){
		Onew_RK_imode_set(0);
		Onew_RK_imode_set(kmode);
		sprintf(help,ONEW_msg_rk_engineswitch,romkan,rkEngineID());
		putmsg(1,"%s %s",Onew_RK_smode(),help);
	}
}
switch_kkserv(){
	char mode[64],help[128],*kanakan;

	if( KanakanP() == 0 )
		kanakan_select();
	if( RomkanN <= 1)
		return;

	if( kanakan = next_kanakan() ){
		sprintf(mode,ONEW_mode_kanakan,kkEngineID());
		sprintf(help,ONEW_msg_kk_engineswitch,kanakan,kkEngineID());
		if( KanakanP()->kk_ready )
			(*KanakanP()->kk_ready)();
		putmsg(1,"%s %s",mode,help);
	}
}

onew_put_kanakanmode(){
	char mode[64];
	sprintf(mode,ONEW_mode_kanakan,kkEngineID());
	Onew_putmode(mode,ONEW_menu_kanakan);
}
/*
 *	let the loader load at least one engine.
 */
extern int RomkanExists;  static int *RkE = &RomkanExists;
extern int KanakanExists; static int *KkE = &KanakanExists;
